
/***********************************************************/
/*                                                         */
/* THIS IS A DEMO PROGRAM FOR USING A C6000 TIMER TO       */
/* DIVIDE A CLOCK										   */
/*                                                         */
/*     JT    ver 1.0  10/04/00   first written             */
/*     PW    ver 1.1  13-5-02    made a divide example     */
/*                               original timer example    */
/***********************************************************/

#include <stdio.h>

/* 
    Description of the bits in the control register:

	BIT     NAME            R/W     VALUE           DESCRIPTION
	 0      func            RW      0x001           1: timer pin, 0: general purpose i/o pin
	 1      i/o             RW      0x002           (used if bit0 is 0) 1: output pin, 0: input pin
	 2      datout			RW      0x004           (used if bit0 is 0 && bit1 is 1) value driven out on TIM pin
	 3      datin           R       0x008           (if bit0 is 0 && bit1 is 0) value on TIM pin
	 4      pwid            RW      0x010           Pulse width. 
	 5      reserved        R       0x020
	 6      go              RW      0x040           1: (if bit7 = 1) zero counter & start counting
	 7      hld             RW      0x080           1: counter allowed to count, 0: counter disabled
	 8      c/p             RW      0x100           1: clock mode, 0: pulse mode
	 9      clksrc			RW      0x200           1: internal clock f=cpu-clock/4, 0: value on TIM pin
	10      inv             RW      0x400           inverts the TINP pin signal 
	11      tstat           R       0x800           Timer status. Value of timer output.
*/

typedef union timer_control
{
	unsigned int c;
	struct
	{
		unsigned int func:1;			/* bit  0 */
		unsigned int invout:1;			/* bit  1 */
		unsigned int datout:1;			/* bit  2 */
		unsigned int datin:1;			/* bit  3 */
		unsigned int pwid:1;			/* bit  4 */
		unsigned int rsvd:1;			/* bit  5 */
		unsigned int go:1;				/* bit  6 */
		unsigned int hld:1;				/* bit  7 */
		unsigned int cp:1;				/* bit  8 */
		unsigned int clksrc:1;			/* bit  9 */
		unsigned int invinp:1;			/* bit 10 */
		unsigned int tstat:1;			/* bit 11 */
	} x;

} CONTROL;

void divide_clock1(int ratio)
{
   volatile unsigned int *timer1_control = (unsigned int *) 0x01980000;
   volatile unsigned int *timer1_period  = (unsigned int *) 0x01980004;

	CONTROL      control;

	control.c        = 0;

	control.x.func   = 1;		/* TOUT is a timer output pin   */
	control.x.invout = 0;		/* Uninverted TSTAT drives TOUT */
	control.x.datout = 0;		/* DATOUT is driven onto TOUT   */
	control.x.datin  = 0;		/* R                            */

	control.x.pwid   = 0;		/* Puls width is 1 clock cycle  */
	control.x.rsvd   = 0;		/* Reserved                     */
	control.x.go     = 1;		/* Reset timer & start counting */
	control.x.hld    = 1;		/* Enable counter               */

	control.x.cp     = 1;		/* TSTAT has 50% duty cycle     */
	control.x.clksrc = 0;		/* Ext. clock source drive TINP */
	control.x.invinp = 0;		/* Don't invert TINP            */


	/* External clock, don't invert TINP pin */
	*timer1_control = (unsigned int) 0x00000001;	/* Halt the timer              */
	*timer1_period  = (unsigned int) ratio/2;		/* Program the period register */
	*timer1_control = (unsigned int) control.c;		/* GO!                         */
}
