/******************************************************************/
/*                                                                */
/*  Flash ROM program for HERON2                                  */
/*                                                                */
/*  This program will either load a *.out file out of flash       */
/*  memory, or if there's no program loaded, try to load a        */
/*  *.out file from FIFO 0.                                       */
/*                                                                */
/*	PW	ver 1.0	    18/07/02	copied from HERON4 and ported       */
/*                                                                */
/******************************************************************/


/* Version */
#define VERSION			1

#include "hsb.h"

/************************************************************/

#define CLKOUT1_RATE 300000000 

#define REFPER (int)((double)CLKOUT1_RATE * 15.625E-6 / 2.0)

extern cregister volatile unsigned int AMR;
extern cregister volatile unsigned int CSR;
extern cregister volatile unsigned int IFR;
extern cregister volatile unsigned int ISR;
extern cregister volatile unsigned int ICR;
extern cregister volatile unsigned int IER;
extern cregister volatile unsigned int ISTP;
extern cregister volatile unsigned int IRP;
extern cregister volatile unsigned int NRP;
extern cregister volatile unsigned int IN;
extern cregister volatile unsigned int OUT;
 

#define FIFO0_EF_MASK 0x1000  /* Bit 12 = 1: Can read one              */

int readfifo = 0;
volatile unsigned int slave_id;

/*
   Copy memory using DMA functions
*/

void dma_copy(unsigned int from, unsigned int to, int n)
{
   volatile unsigned int *dma_ch0_primary_control     = (unsigned int *) 0x01840000;
   volatile unsigned int *dma_ch0_secondary_control   = (unsigned int *) 0x01840008;
   volatile unsigned int *dma_ch0_source_address      = (unsigned int *) 0x01840010;
   volatile unsigned int *dma_ch0_destination_address = (unsigned int *) 0x01840018;
   volatile unsigned int *dma_ch0_transfer_counter    = (unsigned int *) 0x01840020;

   *dma_ch0_primary_control     = 0;                /* Stop/initialise DMA */

   *dma_ch0_source_address      = from;
   *dma_ch0_destination_address = to;
   *dma_ch0_transfer_counter    = n;
   *dma_ch0_secondary_control   = 0x0000a000;
   *dma_ch0_primary_control     = 0x01000051;           /* start dma */

   while ((*dma_ch0_primary_control&0xc)!=0) {}
}

/* 
   Read from FIFO
*/

int read_a_word()
{
   int           data_in;
   volatile int *fifo0    = (int *)0x40000000;
   volatile int *in_flags = (int *)0x00400000;

   while ((*in_flags & FIFO0_EF_MASK) == 0)
   {
		asm("	nop 5");
		asm("	nop 5");
		asm("	nop 5");
		asm("	nop 5");
		asm("	nop 5");
		asm("	nop 5");
		asm("	nop 5");
		asm("	nop 5");
		asm("	nop 5");
		asm("	nop 5");
		asm("	nop 5");
		asm("	nop 5");
		asm("	nop 5");
		asm("	nop 5");
   }

   dma_copy((unsigned int)*fifo0,(unsigned int)&data_in,1); 

   return (data_in);
}

int entry_point;

int boot_program(int modid)
{
   volatile int *dig_io                = (int *)0x00400014;
   volatile int *fifo0    = (int *)0x40000000;
   volatile int *in_flags = (int *)0x00400000;


   unsigned int  block_addr;
   int          i,block_size,read_but_ignore;
   volatile int temp;
   volatile int mod_car;
   volatile int dummy;

   read_but_ignore = 1;

   mod_car = read_a_word() & 0xff;
   if (mod_car == modid)
   {
      read_but_ignore = 0;			/* it's for us                               */
      IER            &= 0x9FFF;		/* Disable timer & host port  interrupt */
	  CSR			 &= (~1);		/* Globally disable interrupts               */
	  ICR		      = 0xfff0;		/* Clear interrupts 4..15    */
   }
   else
   {
      read_but_ignore = 1;            /* it's not for us */
   }

   block_size = read_a_word();
   block_size = block_size >> 2;
   while (block_size != 0)
   {
      block_addr = read_a_word();
      if (read_but_ignore)	/* got to read it, but don't write */
	  {					/* to memory.                      */
         for (i=0;i<block_size;i++)
		 {
            dummy = read_a_word();
		 }
	  }
      else
	  {							/* it's for us! */   
	   for(i=0;i<block_size;i++)
		{
		   while ((*in_flags & FIFO0_EF_MASK) == 0)
   			{
				asm("	nop 5");
 		  	}
                   dma_copy((unsigned int)*fifo0,block_addr+i,1); 
                 }
          }
      block_size = read_a_word();
      block_size = block_size >> 2;
   }
   entry_point = read_a_word();                /* Last word is the entry point */
   *dig_io     = 0xf;

   return (read_but_ignore);
}


main()
{
   volatile unsigned int *flash;
   volatile unsigned int  address, size, magic;

   volatile int      *mod_car_id       = (int *)0x00400018;
   volatile int        *emif_gcr       = (int *)0x01800000;
   volatile int *emif_ce1_timing       = (int *)0x01800004;
   volatile int *emif_ce0_timing       = (int *)0x01800008;
   volatile int *emif_ce2_timing       = (int *)0x01800010;
   volatile int *emif_ce3_timing       = (int *)0x01800014;
   volatile int *emif_sdram_control    = (int *)0x01800018;
   volatile int *emif_sdram_refresh    = (int *)0x0180001c;
   volatile int        *xbus_gcr       = (int *)0x01880000;
   volatile int *dig_io                = (int *)0x00400014;

   int          j, modid;
   volatile unsigned int *ptr;

   /* Program the EMIF */
 
   *emif_gcr           = 0x00003058;	 
   *emif_ce0_timing    = 0x00000030;
   *emif_ce1_timing    = 0x25611522;	/* HERON2 flash rom value; */
   *emif_ce2_timing    = 0x00000030;
   *emif_ce3_timing    = 0x00000030;	
   *emif_sdram_control = 0x07229000;	
   *emif_sdram_refresh = REFPER;	

   /* Program the XBUS */
   *xbus_gcr           = 0x00006000;	



   /* =========================================================== */
   /* First look in flash memory and see if there's a boot stream */
   flash   = (unsigned int *) 0x1402000;
   address = *flash++;
   size    = *flash++;
   magic   = *flash++;
   j       =  1;
   while ((size)&&(magic==0xdeafd00f))
   {
      *dig_io = ~j++;
	  /* Copy this segment to memory */
      dma_copy((unsigned int) flash, address, size);
      flash  += ((size+3)>>2);          /* size is in bytes */
      address = *flash++;
      size    = *flash++;
      magic   = *flash++;
   }

   if (magic==0xdeafd00f)
   {
      *dig_io = 0xf;
      /* We reached the end of the boot stream in flash memory */
      /* The last entry should be the entry point. Read it.    */
      entry_point = *flash++;
      *emif_ce1_timing    = 0x0090420;	/* this bus control register value for non-flash accesses */
      goto jump2userprogram;
   }
   /* ======================================================== */


   /* ========================================================== */
   /* If we're here we didn't find a proper boot stream in flash */
   /* No problem, just wait for the FIFO to find a boot stream.  */
 

   *dig_io = ~VERSION;				/* Indicate our version     */
   modid   = ((*mod_car_id)&0xff);
   ICR     = 0xfff0;				/* Clear interrupts 4..15    */
   CSR     = CSR|1;					/* Globally enable interrupts*/

   slave_id      =  (modid&0x07);
   slave_id     |= ((modid&0xf0)>>1);

   /* Keep hsb data also in a local memory area at fixed  */
   /* address, so that we can find the data whatever the  */
   /* heron module type is. Used by plug-ins, who have a  */
   /* hard time doing hsb.								  */
   ptr = (volatile unsigned int *) DATAADDR;
   ptr[0] = MAGIC;					/* verification	      */
   ptr[1] = RESP_TYPE;				/* response type      */
   ptr[2] = slave_id;				/* sender hsb address */
   ptr[3] = PROCESSOR;				/* module type        */
   ptr[4] = FAMILY;					/* HERON type         */
   ptr[5] = PROCTYPE;				/* Processor type     */
   ptr[6] = VERSION;				/* ROM version		  */
   ptr[7] = 0;						/* Option number      */






   j = 1;
   while (j)
   {
	  if (readfifo)
	  {
         readfifo = 0;
         j        = boot_program(modid);
	  }
      asm("   nop     5");
   }

jump2userprogram:

   asm("   nop     5");
   asm("   ldw     *+b14(_entry_point),a0");
   asm("   nop     5");
   asm("   b.s2    a0");   /* Branch to user program */
   asm("   nop     5");
}


/* This is the Interrupt Service Fetch Packet for RESET.                */
/* It must be programmed at 0 so that when the DMA has finished,        */
/* the first thing that executes is the branch this program.            */

asm("   .global _c_int00");
asm("   .sect   .vectors");

asm("l0:        NOP");
asm("   NOP     ");
asm("   b.s2    _c_int00");
asm("   nop     ");
asm("   nop     ");
asm("   nop     ");
asm("   nop     ");
asm("   nop     ");

asm("   .space  32");	/* l1  */
asm("   .space  32");	/* l2  */
asm("   .space  32");	/* l3  */
asm("   .space  32");	/* l4  */
asm("   .space  32");	/* l5  */
asm("   .space  32");	/* l6  */
asm("   .space  32");	/* l7  */
asm("   .space  32");	/* l8  */
asm("   .space  32");	/* l9  */
asm("   .space  32");	/* l10 */
asm("   .space  32");	/* l11 */
asm("   .space  32");	/* l12 */

asm("l13:        NOP");
asm("   NOP     ");
asm("   NOP     ");
asm("   nop     ");
asm("   nop     ");
asm("   nop     ");
asm("   nop     ");
asm("   nop     ");

#if 1
asm("l14:        NOP");
asm("   NOP     ");
asm("   nop     ");
asm("   nop     ");
asm("   nop     ");
asm("   nop     ");
asm("   nop     ");
asm("   nop     ");

asm("l15:        NOP");
asm("   NOP     ");
asm("   nop     ");
asm("   nop     ");
asm("   nop     ");
asm("   nop     ");
asm("   nop     ");
asm("   nop     ");
#else
asm("   .space  32");	/* l14 */
asm("   .space  32");	/* l15 */
#endif
