/***************************************************************************/
/*  program for programming the flash rom on the HERON4 with a bootloader  */
/*      run under CCS to program the flash                                 */
/*      will also accept a user application to program into the flash too  */
/*									   */
/*    ver 2.0 pw 27-11-00  corrected the EMIF control value for FLASH      */
/*                         and corrected sector erase addresses            */
/*                         and supported flash parts from ST               */
/*									   */
/***************************************************************************/


#include "t.c"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* -- HERON4: 2 Mbyte of flash memory: ------------------ */
#define FLASH_MEMORY_SIZE       0x00080000

/* -- MAGIC word to recognise flash has been programmed:- */
#define MAGIC                   0xdeafd00f

/* -- CLOCK rate of the HERON4 module: ------------------ */
/* -- 132 Mhz is for a HERON4 on a HEPC8! --------------- */
#define CLKOUT1_RATE 132000000

/* -- Derive SDRAM refresh rate from the clock frequency- */
#define REFPER (int)((double)CLKOUT1_RATE * 15.625E-6 / 2.0)

extern cregister volatile unsigned int AMR;

extern FILE        *fin;
extern void         cload(char *fname);
extern unsigned int entry_point;                    /* ENTRY POINT OF MODULE              */

void auto_select_command(void)
{
   volatile unsigned int *flash     =(unsigned int *)0x01400000; /* normaly char */

flash[0x45555] = 0x00aa00aa;
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
flash[0x42aaa] = 0x00550055;
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
flash[0x45555] = 0x00900090;
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
}

void sector_erase_command(int sector_address)
{
   int tempa;
   volatile unsigned int *flash     =(unsigned int *)0x01400000; /* normaly char */

flash[0x45555] = 0x00aa00aa;
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
flash[0x42aaa] = 0x00550055;
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
flash[0x45555] = 0x00800080;
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
flash[0x45555] = 0x00aa00aa;
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
flash[0x42aaa] = 0x00550055;
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
flash[sector_address] = 0x00300030;
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");
asm("   NOP     5");

tempa=flash[sector_address] & 0x00800080;
while(tempa != 0x00800080)
        {
        tempa=flash[sector_address] & 0x00800080;
        }
}

void InitFlash(volatile unsigned int *flash)
{
   int i,manuf_code,dev_code,top,bottom;

   top            = 0;
   bottom         = 0;
   flash[0x45555] = 0x00f000f0; 

   auto_select_command();
   manuf_code = flash[0];
   dev_code   = flash[1];
   printf("manuf_code = %08x\n",manuf_code);
   printf("dev_code   = %08x\n",dev_code);
   if(manuf_code == 0x00010001)
   {
        printf("Both parts are AMD.\n");
        if(dev_code == 0x225b225b)
        {
			bottom = 1;
            printf("Both parts are Am29LV800BA - Bottom Boot Block.\n");
        }
        else
        if(dev_code == 0x22da22da)
        {
			top = 1;
            printf("Both parts are Am29LV800BT - Top Boot Block.\n");
        }
        else
        {
            printf("Unknown AMD devices.\n");
            exit(-1);
        }
   }
   else
   if(manuf_code == 0x00040004)
   {
        printf("Both parts are Fujitsu.\n");
        if(dev_code == 0x225b225b)
        {
			bottom = 1;
            printf("Both parts are MBM29LV800BA - Bottom Boot Block.\n");
        }
        else
        if(dev_code == 0x22da22da)
        {
			top = 1;
            printf("Both parts are MBM29LV800TA - Top Boot Block.\n");
        }
        else
        {
            printf("Unknown Fujitsu devices.\n");
            exit(-1);
        }
   }
   else
   if(manuf_code == 0x00200020)
   {
        printf("Both parts are ST.\n");
        if(dev_code == 0x005b005b)
        {
			bottom = 1;
            printf("Both parts are M29W800B - Bottom Boot Block.\n");
        }
        else
        if(dev_code == 0x00d700d7)
        {
			top = 1;
            printf("Both parts are M29W800T - Top Boot Block.\n");
        }
        else
        {
            printf("Unknown ST devices.\n");
            exit(-1);
        }
   }
   else
   {
        printf("Unknown manufacturer codes.\n");
        exit(-1);
   }

   flash[0x45555] = 0x00f000f0;    /* Resets command sequence */

   if (bottom)
   {
      sector_erase_command(0x42000);
      printf("Erased sector %08x\n", 0x42000);
      flash[0x40555] = 0x00f000f0;
      sector_erase_command(0x43000);
      printf("Erased sector %08x\n", 0x43000);
      flash[0x40555] = 0x00f000f0;
      sector_erase_command(0x44000);
      printf("Erased sector %08x\n", 0x44000);
      flash[0x40555] = 0x00f000f0;
   }
   for (i=0; i<FLASH_MEMORY_SIZE; i+=0x8000)
   {
      sector_erase_command(i);
      printf("Erased sector %08x\n", i);
      flash[0x40555] = 0x00f000f0; 
   }
   if (top)
   {
      sector_erase_command(0x3C000);
      printf("Erased sector %08x\n", 0x3C000);
      flash[0x40555] = 0x00f000f0;
      sector_erase_command(0x3D000);
      printf("Erased sector %08x\n", 0x3D000);
      flash[0x40555] = 0x00f000f0;
      sector_erase_command(0x3E000);
      printf("Erased sector %08x\n", 0x3E000);
      flash[0x40555] = 0x00f000f0;
   }

}
  
unsigned char *boot_array;
unsigned int   top = 0;
int            bootloader;
unsigned int   app = 0x2000;

main()
{
   volatile unsigned int *flash     =(unsigned int *)0x01400000; /* normaly char */
   volatile int        *emif_gcr       = (int *)0x01800000;
   volatile int *emif_ce1_timing       = (int *)0x01800004;
   volatile int *emif_ce0_timing       = (int *)0x01800008;
   volatile int *emif_ce2_timing       = (int *)0x01800010;
   volatile int *emif_ce3_timing       = (int *)0x01800014;
   volatile int *emif_sdram_control    = (int *)0x01800018;
   volatile int *emif_sdram_refresh    = (int *)0x0180001c;

   volatile int i;
   volatile int tempa,tempb;
   void wait_100(void);
   void wait_20ms(void);
   short getw(FILE *);
   char coff_filename[30];
   char *dunno;
   int word_data;

   AMR = 0x0;

   /* Program the EMIF */
   *emif_gcr           = 0x000030f8;	
   *emif_ce0_timing    = 0x00000030;	
   *emif_ce1_timing    = 0x62701920;	/* slow accesses for flash */
   *emif_ce2_timing    = 0x00000030;	
   *emif_ce3_timing    = 0x00000040;	
   *emif_sdram_control = 0x07114000;	
   *emif_sdram_refresh = REFPER;	

   boot_array = (unsigned char *)malloc(FLASH_MEMORY_SIZE * sizeof(unsigned char));
   if (boot_array == NULL)
   {
	printf("Couldn't allocate memory for boot_array\n");
	exit(-1);
   }

   for (i=0; i<FLASH_MEMORY_SIZE; i++)
   {
	boot_array[i] = 0xff;
   }

   /* First, load the bootloader: flash1.out or romboot.out */
   printf("Enter boot loader COFF filename:\n");
   dunno = gets(coff_filename);
   if (dunno == NULL)
   {
      printf("error\n");
      exit(-1);
   }

   if ((fin = fopen(coff_filename, "rb")) == NULL)
   {
     printf("Can't open file %s ",coff_filename);
     exit (1);
   }

   bootloader = 1;
   cload(coff_filename);

   fclose(fin);

   /* Second, load the user program */

   printf("Enter application COFF filename:\n");
   dunno = gets(coff_filename);
   if (dunno == NULL)
   {
      printf("error\n");
      exit(-1);
   }

   /* If user responded return only then bypass second file read */
   if (coff_filename[0]!=0)
   {
      if ((fin = fopen(coff_filename, "rb")) == NULL)
      {
	printf("Can't open file %s ",coff_filename);
	exit (1);
      }

      bootloader = 0;
      cload(coff_filename);

      fclose(fin);

      /* Fill in entry point for app */
      i = (app+3)/4;
      ((unsigned int *)boot_array)[i++] = 0;
      ((unsigned int *)boot_array)[i++] = 0;
      ((unsigned int *)boot_array)[i++] = MAGIC;
      ((unsigned int *)boot_array)[i++] = entry_point;
      printf("Entry point for application is 0x%08x.\n", entry_point);
   }

   /* Now program the flash... */

   /* Initialise the flash */
   InitFlash(flash);

   printf("Programming flash (0-0x%08x).\n", top);
   for (i=0; i<FLASH_MEMORY_SIZE; i+=4)        
   {
        if ((i & 0x7ff) == 0)
        {
            printf("programming at 0x%08x\n", i);
        }
        flash[0x45555] = 0x00aa00aa;
        wait_100();
        flash[0x42aaa] = 0x00550055;
        wait_100();
        flash[0x45555] = 0x00a000a0;
        wait_100();
        word_data = (boot_array[i] & 0xff) | ((boot_array[i+1] & 0xff) << 8) | ((boot_array[i+2] & 0xff) << 16) | ((boot_array[i+3] & 0xff) << 24);

        flash[i>>2] = word_data;
        wait_100();
                
        tempa = flash[i>>2] & 0x00800080;
        asm("   nop");
        asm("   nop");
        asm("   nop");
        asm("   nop");

        tempb = word_data & 0x00800080;
                
        while(tempa != tempb)
        {
           tempa = flash[i>>2] & 0x00800080;
        }
        /* wait_20ms(); */
   }

   printf("Finished\n");
}

void wait_100(void)
{
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
return;
}

void wait_20ms(void)
{
int z;
for(z=0;z<20000;z++)
        {
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
asm("   nop");
        }
return;
}

short getw(FILE *fpr)
{
short data;
data = 0xff & getc(fpr);
data =  data | ((0xff & getc(fpr)) << 8);
return(data);
}

extern char secname[256];

int mem_write(unsigned char *buf, int size, unsigned int address, int c)
{
   int i;

   if (bootloader)
   {
      printf("%-8s: size %5d, address 0x%08x\n", secname, size, address);

      if (address<=0x10000) /* internal program memory */
      {
	 for (i=0; i<size; i++) boot_array[address+i] = buf[i];
	 if (address>top) top = address;
      }
      else
      if ((address>= 0x1400000)&&(address<0x1480000))
      {
	 address -= 0x1400000;
	 for (i=0; i<size; i++) boot_array[address+i] = buf[i];
	 if (address>top) top = address;
      }
      else
      {
	 printf("WARNING: cannot store in flash memory!\n");
      }
   }
   else
   {
      printf("%-8s: size %5d, address 0x%08x, flash at 0x%08x\n", secname, size, address, app);
	if (app+12+size>FLASH_MEMORY_SIZE)
	{
	   printf("WARNING: cannot program this section: program too large. [%x:%x]\n", app+12+size, FLASH_MEMORY_SIZE);
	   exit  (-1);
	}
	else
	{
         i = (app+3)/4;
         ((unsigned int *)boot_array)[i++] = address;
         ((unsigned int *)boot_array)[i++] = size;
         ((unsigned int *)boot_array)[i++] = MAGIC;
         app += 12;
         for (i=0; i<size; i++) boot_array[app++] = buf[i];
         app += 3;
         app &= (~0x3);
	}
 
   }
   return 1;
}

int set_reloc_amount(void)
{
   printf("SETRELOC\n");
   return 1;
}

void lookup_sym(long indx, void *sym, void *aux)
{
   printf("LOOKUPSYM %d %x %x\n", indx, sym, aux);
}

int load_syms(int need_reloc)
{
   printf("LOADSYMS %d\n", need_reloc);
   return 0;
}

void *myalloc(unsigned int size)
{
   return malloc(size);
}

void *mralloc(void *p, unsigned int size)
{
   return realloc(p, size);
}

void load_msg()
{
   printf("LOADMSG\n");
}


